#!/usr/bin/env bash

function bd_check_disk_uuid(){
    write_file ${split_line}
    write_file "[ check disk uuid ]"

    if [ ! -r /etc/fstab ]; then
        write_file "\tCRITICAL: /etc/fstab doesn't exist or cannot be read."
        return ${RETURN_CRITICAL}
    fi

    ret=0
    grep -vE "^#|^;|^$" /etc/fstab | grep -i "^UUID" | awk '{ print $2 }' | grep -q "^/$"
    if [ $? -ne 0 ];then
        write_file "\tWARNING: in /etc/fstab, the root partition is not mounted using UUID."
        ret=1
    fi

    # root uses the uuid to mount
    if [[ "${distro}" == "CentOS" || "${distro}" == "Baidu" || "${distro}" == "CentOS Stream" || 
      "${distro}" == "Rocky" || "${distro}" == "AlmaLinux" || "${distro}" == "rhel" ]]; then
        command -v grubby 1>/dev/null 2>&1
        if [ $? -ne 0 ]; then
            write_file "\tWARNING: can't find the grubby command on your system."
            write_file "\tWARNING: please confirm to mount the root partition with UUID."
            return ${RETURN_WARNING}
        fi
        grubby --info=DEFAULT | grep "root" | grep "UUID" 1>/dev/null 2>&1
        if [ $? -ne 0 ]; then
            write_file "\tWARNING: in grub file: ${s_grub_file}, your system doesn't mount the root \
partition with UUID."

            [ ${ret} -eq 1 ] || ret=1
        fi
    fi
    
    if [[ "${distro}" == "Ubuntu" || "${distro}" == "Debian" ]]; then
        if [ -e ${s_grub_file} ]; then
            grep -v "^#" "${s_grub_file}" | grep "$(uname -r)" | grep "root" |\
                    awk -F "=" '{print $2}' | awk '{print $1}' | grep -q "UUID"
            if [ $? -ne 0 ]; then
                write_file "\tWARNING: in grub file: ${s_grub_file}, your system doesn't mount \
the root partition with UUID." 

                [ ${ret} -eq 1 ] || ret=1
            fi
        fi
    fi
    return ${ret}
}

function bd_check_disk_size(){
    write_file ${split_line}
    write_file "[ check disk size ]"

    command -v fdisk 1>/dev/null 2>&1
    if [ $? -ne 0 ]; then
        write_file "\tWARNING: can't find the fdisk command on your system."
        write_file "\tWARNING: please confirm that the disk size is not exceed 500GB."
        return ${RETURN_WARNING}
    fi

    disk_size=$(fdisk -l 2>/dev/null |awk '/^Disk.*bytes/&&/\/dev/{ printf "%d",$3;print "" }' | head -n 1)
    size_unit=$(fdisk -l 2>/dev/null |awk '/^Disk.*bytes/&&/\/dev/{ printf $4 ;print "" }' |\
        awk -F ',' '{print $1}' | head -n 1)
    if [ $size_unit == "TB" ]; then
        disk_size=$((disk_size*1024))
    elif [ $size_unit == "MB" ]; then
        disk_size=$((disk_size/1024))
    fi

    if [ "${disk_size}" -le 500 ]; then
        return ${RETURN_SUCCESS}
    else
        write_file "\tWARNING: the disk size is ${disk_size}GB."
        write_file "\tWARNING: the size of the disk on your system must be less than 500G."
        return ${RETURN_WARNING}
    fi
}

function bd_check_partition_type(){
    write_file ${split_line}
    write_file "[ check disk partition ]"

    command -v parted 1>/dev/null 2>&1
    if [ $? -ne 0 ]; then
        write_file "\tWARNING: can't find parted command."
        write_file "\tWARNING: please make sure that your system uses MBR partition."
        return ${RETURN_WARNING}
    fi

    ret=0
    partition_type=
    if [ ${boot_mode} = "UEFI" ]; then
        parted -l 2>/dev/null | grep "Partition Table" | grep -q "gpt"
        if [ $? -ne 0 ]; then 
            ret=1
            partition_type="gpt"
        fi
    fi

    if [ ${boot_mode} = "BIOS(Legacy)" ]; then
        parted -l 2>/dev/null | grep "Partition Table" | grep -q "msdos"
        if [ $? -ne 0 ]; then 
            ret=1
            partition_type="msdos"
        fi
    fi

    if [ ${ret} -eq 0 ]; then
        return ${RETURN_SUCCESS}
    else
        p_type=$(parted -l 2>/dev/null | grep -i "Partition Table" | head -n 1 | awk '{print $3}') 1>/dev/null 2>&1
        if [ $? -eq 0 ]; then
            write_file "\tWARNING: the Partition Table on your disk is ${p_type}."
            write_file "\tWARNING: we recommend using ${partition_type} partition."
            return ${RETURN_WARNING}
        else
            write_file "\tWARNING: can't find Partition Table type."
            write_file "\tWARNING: please confirm that the partition table is ${partition_type}."
            return ${RETURN_WARNING}
        fi
    fi
}

function bd_check_lvm(){
    write_file ${split_line}
    write_file "[ check lvm ]"

    disk=$(fdisk -l 2>/dev/null | awk '/^Disk.*bytes/&&/\/dev/{print $2}' | awk -F ':' '{print $1}' | head -n 1)
    if [ -n ${disk} ]; then
        lsblk -l | grep -q "lvm" 1>/dev/null 2>&1
        if [ $? -eq 0 ]; then
            write_file "\tWARNING: your system uses LVM partition."
            write_file "\tWARNING: please use normal disk partition."
            return ${RETURN_WARNING}
        else
            return ${RETURN_SUCCESS}
        fi
    else
        write_file "\tWARNING: can't find the system partition."
        write_file "\tWARNING: please confirm that your system is not using LVM."
        return ${RETURN_WARNING}
    fi
}